---
---
<!DOCTYPE html>
<html>
<head>
<title>Posts</title>
</head>
<body>
<div class="index">
<h1>Posts</h1>
<ul class="posts">
{% for post in site.posts %}
<li>
<a href="{{ post.url }}">{{ post.title }}</a>
{% if post.excerpt != '' %}<div class="excerpt">{{ post.excerpt }}</div>{% endif %}
</li>
{% endfor %}
</ul>
</div>
</body>
</html>
