<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
<title>{{ page.title }}</title>
</head>
<body>
<div class="page-content">
{{ content }}
</div>
{% if page.asciidoc %}
<aside class="page-toc">
{{ page.document | tocify_asciidoc: 2 }}
</aside>
{% endif %}
</body>
</html>
