<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
<title>{{ page.title }}</title>
</head>
<body>
<div class="page-content">
{{ content }}
</div>
<footer>
<p>Footer for docs layout.</p>
</footer>
</body>
</html>
