<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
<title>{{ page.title }}</title>
</head>
<body>
<article class="post">
<header class="post-header">
<h1 class="post-title">{{ page.title }}</h1>
<p class="post-details">
<time class="date-published" datetime="{{ page.date | date_to_xmlschema }}">{{ page.date | date: "%B %d, %Y" }}</time>
</p>
</header>
<div class="post-content">
{{ content }}
</div>
</article>
<footer>
<p>Footer for post layout.</p>
</footer>
</body>
</html>
